@@ -32,7 +32,7 @@ import tenacity
 
 from oslo_messaging._drivers import base
 from oslo_messaging._drivers import common as driver_common
-from oslo_messaging._drivers import kafka_options
+from oslo_messaging._drivers.kafka_driver import kafka_options
 from oslo_messaging._drivers import pool as driver_pool
 from oslo_messaging._i18n import _LE
 from oslo_messaging._i18n import _LW
@@ -124,7 +124,7 @@ class Connection(object):
         self.producer = None
         self.producer_lock = threading.Lock()
         self.consumer = None
-        self.consumer_timeout = float(driver_conf.kafka_consumer_timeout)
+        self.consumer_timeout = driver_conf.kafka_consumer_timeout
         self.max_fetch_bytes = driver_conf.kafka_max_fetch_bytes
         self.group_id = driver_conf.consumer_group
         self.url = url
@@ -191,7 +191,7 @@ class Connection(object):
         """
 
         def _raise_timeout(exc):
-            raise driver_common.Timeout(exc.message)
+            raise driver_common.Timeout(str(exc))
 
         timer = driver_common.DecayingTimer(duration=timeout)
         timer.start()
@@ -276,6 +276,11 @@ class KafkaListener(base.PollStyleListener):
         self.conn = conn
         self.incoming_queue = []
 
+        # FIXME(sileht): We do a first poll to ensure we topics are created
+        # This is a workaround mainly for functional tests, in real life
+        # this is fine if topics are not created synchroneously
+        self.poll(5)
+
     @base.batch_poll_helper
     def poll(self, timeout=None):
         while not self._stopped.is_set():
@@ -304,10 +309,10 @@ class KafkaDriver(base.BaseDriver):
 
     def __init__(self, conf, url, default_exchange=None,
                  allowed_remote_exmods=None):
+        conf = kafka_options.register_opts(conf, url)
         super(KafkaDriver, self).__init__(
             conf, url, default_exchange, allowed_remote_exmods)
 
-        kafka_options.register_opts(conf)
         # the pool configuration properties
         max_size = self.conf.oslo_messaging_kafka.pool_size
         min_size = self.conf.oslo_messaging_kafka.conn_pool_min_size
