@@ -20,7 +20,6 @@ import argparse
 import logging
 import uuid
 
-from debtcollector import renames
 from oslo_config import cfg
 from oslo_utils import timeutils
 import six
@@ -55,6 +54,10 @@ _notifier_opts = [
                                       group='DEFAULT')
                 ],
                 help='AMQP topic used for OpenStack notifications.'),
+    cfg.IntOpt('retry', default=-1,
+               help='The maximum number of attempts to re-send a notification '
+                    'message which failed to be delivered due to a '
+                    'recoverable error. 0 - No retry, -1 - indefinite'),
 ]
 
 _LOG = logging.getLogger(__name__)
@@ -155,7 +158,7 @@ def get_notification_transport(conf, url=None,
 
     :param conf: the user configuration
     :type conf: cfg.ConfigOpts
-    :param url: a transport URL
+    :param url: a transport URL, see :py:class:`transport.TransportURL`
     :type url: str or TransportURL
     :param allowed_remote_exmods: a list of modules which a client using this
                                   transport will deserialize remote exceptions
@@ -168,8 +171,8 @@ def get_notification_transport(conf, url=None,
                        group='oslo_messaging_notifications')
     if url is None:
         url = conf.oslo_messaging_notifications.transport_url
-    return msg_transport.get_transport(conf, url,
-                                       allowed_remote_exmods, aliases)
+    return msg_transport._get_transport(conf, url,
+                                        allowed_remote_exmods, aliases)
 
 
 class Notifier(object):
@@ -205,7 +208,7 @@ class Notifier(object):
         notifier = notifier.Notifier(transport,
                                      'compute.host',
                                      driver='messaging',
-                                     topic='notifications')
+                                     topics=['notifications'])
 
     Notifier objects are relatively expensive to instantiate (mostly the cost
     of loading notification drivers), so it is possible to specialize a given
@@ -215,13 +218,8 @@ class Notifier(object):
         notifier.info(ctxt, event_type, payload)
     """
 
-    @renames.renamed_kwarg('topic', 'topics',
-                           message="Please use topics instead of topic",
-                           version='4.5.0',
-                           removal_version='5.0.0')
     def __init__(self, transport, publisher_id=None,
-                 driver=None, topic=None,
-                 serializer=None, retry=None,
+                 driver=None, serializer=None, retry=None,
                  topics=None):
         """Construct a Notifier object.
 
@@ -232,8 +230,6 @@ class Notifier(object):
         :type publisher_id: str
         :param driver: a driver to lookup from oslo_messaging.notify.drivers
         :type driver: str
-        :param topic: the topic which to send messages on
-        :type topic: str
         :param serializer: an optional entity serializer
         :type serializer: Serializer
         :param retry: connection retries configuration (used by the messaging
@@ -251,15 +247,16 @@ class Notifier(object):
 
         self.transport = transport
         self.publisher_id = publisher_id
-        self.retry = retry
+        if retry is not None:
+            self.retry = retry
+        else:
+            self.retry = conf.oslo_messaging_notifications.retry
 
         self._driver_names = ([driver] if driver is not None else
                               conf.oslo_messaging_notifications.driver)
 
         if topics is not None:
             self._topics = topics
-        elif topic is not None:
-            self._topics = [topic]
         else:
             self._topics = conf.oslo_messaging_notifications.topics
         self._serializer = serializer or msg_serializer.NoOpSerializer()
@@ -425,6 +422,14 @@ class Notifier(object):
         """
         self._notify(ctxt, event_type, payload, 'SAMPLE')
 
+    def is_enabled(self):
+        """Check if the notifier will emit notifications anywhere.
+
+        :return: false if the driver of the notifier is set only to noop, true
+                 otherwise
+        """
+        return self._driver_mgr.names() != ['noop']
+
 
 class _SubNotifier(Notifier):
 
