@@ -20,12 +20,12 @@ methods which may be invoked remotely by clients over a given transport.
 To create an RPC server, you supply a transport, target and a list of
 endpoints.
 
-A transport can be obtained simply by calling the get_transport() method::
+A transport can be obtained simply by calling the get_rpc_transport() method::
 
-    transport = messaging.get_transport(conf)
+    transport = messaging.get_rpc_transport(conf)
 
 which will load the appropriate transport driver according to the user's
-messaging configuration. See get_transport() for more details.
+messaging configuration. See get_rpc_transport() for more details.
 
 The target supplied when creating an RPC server expresses the topic, server
 name and - optionally - the exchange to listen on. See Target for more details
@@ -47,12 +47,8 @@ Server will send the returned value back to the requesting client via the
 transport.
 
 The executor parameter controls how incoming messages will be received and
-dispatched. By default, the most simple executor is used - the blocking
-executor.  This executor processes inbound RPC requests on the server's thread,
-blocking it from processing additional requests until it finishes with the
-current request. This includes time spent sending the reply message to the
-transport if the method returns a result.  Refer to the Executor documentation
-for descriptions of the other types of executors.
+dispatched. Refer to the Executor documentation for descriptions of the types
+of executors.
 
 *Note:* If the "eventlet" executor is used, the threading and time library need
 to be monkeypatched.
@@ -77,6 +73,13 @@ complete after the Server has been stopped.
 
 A simple example of an RPC server with multiple endpoints might be::
 
+    # NOTE(changzhi): We are using eventlet executor and
+    # time.sleep(1), therefore, the server code needs to be
+    # monkey-patched.
+
+    import eventlet
+    eventlet.monkey_patch()
+
     from oslo_config import cfg
     import oslo_messaging
     import time
@@ -98,14 +101,14 @@ A simple example of an RPC server with multiple endpoints might be::
         def test(self, ctx, arg):
             return arg
 
-    transport = oslo_messaging.get_transport(cfg.CONF)
+    transport = oslo_messaging.get_rpc_transport(cfg.CONF)
     target = oslo_messaging.Target(topic='test', server='server1')
     endpoints = [
         ServerControlEndpoint(None),
         TestEndpoint(),
     ]
     server = oslo_messaging.get_rpc_server(transport, target, endpoints,
-                                           executor='blocking')
+                                           executor='eventlet')
     try:
         server.start()
         while True:
@@ -127,6 +130,8 @@ __all__ = [
 import logging
 import sys
 
+from debtcollector.updating import updated_kwarg_default_value
+
 from oslo_messaging._i18n import _LE
 from oslo_messaging.rpc import dispatcher as rpc_dispatcher
 from oslo_messaging import server as msg_server
@@ -177,6 +182,15 @@ class RPCServer(msg_server.MessageHandlingServer):
                 del failure
 
 
+@updated_kwarg_default_value('access_policy', None,
+                             rpc_dispatcher.DefaultRPCAccessPolicy,
+                             message='access_policy defaults to '
+                                     'LegacyRPCAccessPolicy which '
+                                     'exposes private methods. Explicitly '
+                                     'set access_policy to '
+                                     'DefaultRPCAccessPolicy or '
+                                     'ExplicitRPCAccessPolicy.',
+                             version='?')
 def get_rpc_server(transport, target, endpoints,
                    executor='blocking', serializer=None, access_policy=None):
     """Construct an RPC server.
@@ -187,8 +201,8 @@ def get_rpc_server(transport, target, endpoints,
     :type target: Target
     :param endpoints: a list of endpoint objects
     :type endpoints: list
-    :param executor: name of a message executor - for example
-                     'eventlet', 'blocking'
+    :param executor: name of message executor - available values are
+                     'eventlet' and 'threading'
     :type executor: str
     :param serializer: an optional entity serializer
     :type serializer: Serializer
