@@ -17,6 +17,7 @@ import os
 import select
 import shlex
 import shutil
+from six.moves import mock
 import socket
 import subprocess
 import sys
@@ -52,7 +53,8 @@ _proton = importutils.try_import("proton")
 CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton
                  and getattr(_proton.SASL, "extended", lambda: False)())
 # same with SSL
-SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
+# SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
+SSL_ENABLED = False
 
 LOG = logging.getLogger(__name__)
 
@@ -710,7 +712,7 @@ class TestCyrusAuthentication(test_utils.BaseTestCase):
         # the temp dir after the first test is run
         os.makedirs(cls._conf_dir)
         db = os.path.join(cls._conf_dir, 'openstack.sasldb')
-        _t = "echo secret | saslpasswd2 -c -p -f ${db} joe"
+        _t = "echo secret | saslpasswd2 -c -p -f ${db} -u myrealm joe"
         cmd = Template(_t).substitute(db=db)
         try:
             subprocess.check_call(args=cmd, shell=True)
@@ -743,7 +745,7 @@ mech_list: ${mechs}
         _dir = TestCyrusAuthentication._conf_dir
         self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
                                   sasl_mechanisms=_mechs,
-                                  user_credentials=["\0joe\0secret"],
+                                  user_credentials=["\0joe@myrealm\0secret"],
                                   sasl_config_dir=_dir,
                                   sasl_config_name="openstack")
         self._broker.start()
@@ -756,45 +758,44 @@ mech_list: ${mechs}
             self._broker = None
         super(TestCyrusAuthentication, self).tearDown()
 
-    def test_authentication_ok(self):
-        """Verify that username and password given in TransportHost are
-        accepted by the broker.
-        """
-        addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
-                                            self._broker.port)
+    def _authentication_test(self, addr, retry=None):
         url = oslo_messaging.TransportURL.parse(self.conf, addr)
         driver = amqp_driver.ProtonDriver(self.conf, url)
         target = oslo_messaging.Target(topic="test-topic")
         listener = _ListenerThread(
             driver.listen(target, None, None)._poll_style_listener, 1)
-        rc = driver.send(target, {"context": True},
-                         {"method": "echo"}, wait_for_reply=True)
-        self.assertIsNotNone(rc)
-        listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
-        driver.cleanup()
+        try:
+            rc = driver.send(target, {"context": True},
+                             {"method": "echo"}, wait_for_reply=True,
+                             retry=retry)
+            self.assertIsNotNone(rc)
+            listener.join(timeout=30)
+            self.assertFalse(listener.isAlive())
+        finally:
+            driver.cleanup()
+
+    def test_authentication_ok(self):
+        """Verify that username and password given in TransportHost are
+        accepted by the broker.
+        """
+        addr = "amqp://joe@myrealm:secret@%s:%d" % (self._broker.host,
+                                                    self._broker.port)
+        self._authentication_test(addr)
 
     def test_authentication_failure(self):
         """Verify that a bad password given in TransportHost is
         rejected by the broker.
         """
-        addr = "amqp://joe:badpass@%s:%d" % (self._broker.host,
-                                             self._broker.port)
-        url = oslo_messaging.TransportURL.parse(self.conf, addr)
-        driver = amqp_driver.ProtonDriver(self.conf, url)
-        target = oslo_messaging.Target(topic="test-topic")
-        _ListenerThread(
-            driver.listen(target, None, None)._poll_style_listener, 1)
+        addr = "amqp://joe@myrealm:badpass@%s:%d" % (self._broker.host,
+                                                     self._broker.port)
         try:
-            driver.send(target, {"context": True}, {"method": "echo"},
-                        wait_for_reply=True, retry=2)
+            self._authentication_test(addr, retry=2)
         except oslo_messaging.MessageDeliveryFailure as e:
             # verify the exception indicates the failure was an authentication
             # error
             self.assertTrue('amqp:unauthorized-access' in str(e))
         else:
             self.assertIsNone("Expected authentication failure")
-        driver.cleanup()
 
     def test_authentication_bad_mechs(self):
         """Verify that the connection fails if the client's SASL mechanisms do
@@ -802,40 +803,41 @@ mech_list: ${mechs}
         """
         self.config(sasl_mechanisms="EXTERNAL ANONYMOUS",
                     group="oslo_messaging_amqp")
-        addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
-                                            self._broker.port)
-        url = oslo_messaging.TransportURL.parse(self.conf, addr)
-        driver = amqp_driver.ProtonDriver(self.conf, url)
-        target = oslo_messaging.Target(topic="test-topic")
-        _ListenerThread(
-            driver.listen(target, None, None)._poll_style_listener, 1)
+        addr = "amqp://joe@myrealm:secret@%s:%d" % (self._broker.host,
+                                                    self._broker.port)
         self.assertRaises(oslo_messaging.MessageDeliveryFailure,
-                          driver.send,
-                          target, {"context": True},
-                          {"method": "echo"},
-                          wait_for_reply=True,
+                          self._authentication_test,
+                          addr,
                           retry=0)
-        driver.cleanup()
 
     def test_authentication_default_username(self):
         """Verify that a configured username/password is used if none appears
         in the URL.
+        Deprecated: username password deprecated in favor of transport_url
         """
         addr = "amqp://%s:%d" % (self._broker.host, self._broker.port)
-        self.config(username="joe",
+        self.config(username="joe@myrealm",
                     password="secret",
                     group="oslo_messaging_amqp")
-        url = oslo_messaging.TransportURL.parse(self.conf, addr)
-        driver = amqp_driver.ProtonDriver(self.conf, url)
-        target = oslo_messaging.Target(topic="test-topic")
-        listener = _ListenerThread(
-            driver.listen(target, None, None)._poll_style_listener, 1)
-        rc = driver.send(target, {"context": True},
-                         {"method": "echo"}, wait_for_reply=True)
-        self.assertIsNotNone(rc)
-        listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
-        driver.cleanup()
+        self._authentication_test(addr)
+
+    def test_authentication_default_realm(self):
+        """Verify that default realm is used if none present in username"""
+        addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
+                                            self._broker.port)
+        self.config(sasl_default_realm="myrealm",
+                    group="oslo_messaging_amqp")
+        self._authentication_test(addr)
+
+    def test_authentication_ignore_default_realm(self):
+        """Verify that default realm is not used if realm present in
+        username
+        """
+        addr = "amqp://joe@myrealm:secret@%s:%d" % (self._broker.host,
+                                                    self._broker.port)
+        self.config(sasl_default_realm="bad-realm",
+                    group="oslo_messaging_amqp")
+        self._authentication_test(addr)
 
 
 @testtools.skipUnless(pyngus, "proton modules not present")
@@ -1418,16 +1420,24 @@ class TestMessageRetransmit(_AmqpBrokerTestCase):
                                                  None,
                                                  None)._poll_style_listener,
                                    1)
-        rc = driver.send(target, {"context": "whatever"},
-                         {"method": "echo", "id": "blah"},
-                         wait_for_reply=True,
-                         retry=2)  # initial send + up to 2 resends
-        self.assertIsNotNone(rc)
-        self.assertEqual(0, self._nack_count)
-        self.assertEqual(rc.get('correlation-id'), 'blah')
-        listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
-        driver.cleanup()
+        try:
+            rc = driver.send(target, {"context": "whatever"},
+                             {"method": "echo", "id": "blah"},
+                             wait_for_reply=True,
+                             retry=2)  # initial send + up to 2 resends
+        except Exception:
+            # Some test runs are expected to raise an exception,
+            # clean up the listener since no message was received
+            listener.kill(timeout=30)
+            raise
+        else:
+            self.assertIsNotNone(rc)
+            self.assertEqual(0, self._nack_count)
+            self.assertEqual(rc.get('correlation-id'), 'blah')
+            listener.join(timeout=30)
+        finally:
+            self.assertFalse(listener.isAlive())
+            driver.cleanup()
 
     def test_released(self):
         # should retry and succeed
@@ -1470,11 +1480,10 @@ class TestSSL(test_utils.BaseTestCase):
         conf['c_name'] = 'client.com'
         self._ssl_config = conf
         ssl_setup = [
-            # create self-signed CA key and certificate:
-            Template('openssl genrsa -out ${ca_key} 2048').substitute(conf),
-            Template('openssl req -x509 -key ${ca_key} -subj'
-                     ' "/CN=Trusted.CA.com" -out'
-                     ' ${ca_cert}').substitute(conf),
+            # create self-signed CA certificate:
+            Template('openssl req -x509 -nodes -newkey rsa:2048'
+                     ' -subj "/CN=Trusted.CA.com" -keyout ${ca_key}'
+                     ' -out ${ca_cert}').substitute(conf),
             # create Server key and certificate:
             Template('openssl genrsa -out ${s_key} 2048').substitute(conf),
             Template('openssl req -new -key ${s_key} -subj /CN=${s_name}'
@@ -1532,6 +1541,32 @@ class TestSSL(test_utils.BaseTestCase):
         self.assertFalse(listener.isAlive())
         driver.cleanup()
 
+    @mock.patch('ssl.get_default_verify_paths')
+    def test_server_ok_with_ssl_set_in_transport_url(self, mock_verify_paths):
+        # test client authenticates server
+        self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
+                                  sock_addr=self._ssl_config['s_name'],
+                                  ssl_config=self._ssl_config)
+        url = oslo_messaging.TransportURL.parse(
+            self.conf, "amqp://%s:%d?ssl=1" % (self._broker.host,
+                                               self._broker.port))
+        self._broker.start()
+        mock_verify_paths.return_value = mock.Mock(
+            cafile=self._ssl_config['ca_cert'])
+        driver = amqp_driver.ProtonDriver(self.conf, url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _ListenerThread(
+            driver.listen(target, None, None)._poll_style_listener, 1)
+
+        driver.send(target,
+                    {"context": "whatever"},
+                    {"method": "echo", "a": "b"},
+                    wait_for_reply=True,
+                    timeout=30)
+        listener.join(timeout=30)
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
     def test_bad_server_fail(self):
         # test client does not connect to invalid server
         self._ssl_config['s_cert'] = self._ssl_config['bad_cert']
