@@ -20,7 +20,7 @@ from oslo_config import cfg
 from six import moves
 
 import oslo_messaging
-from oslo_messaging._drivers import kafka_options
+from oslo_messaging._drivers.kafka_driver import kafka_options
 from oslo_messaging._drivers.zmq_driver import zmq_options
 from oslo_messaging.notify import notifier
 from oslo_messaging.tests import utils as test_utils
@@ -294,7 +294,9 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
         if not self.url:
             self.skipTest("No transport url configured")
 
-        zmq_options.register_opts(conf)
+        transport_url = oslo_messaging.TransportURL.parse(conf, self.url)
+
+        zmq_options.register_opts(conf, transport_url)
 
         zmq_matchmaker = os.environ.get('ZMQ_MATCHMAKER')
         if zmq_matchmaker:
@@ -322,7 +324,7 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
         self.config(use_dynamic_connections=zmq_use_dynamic_connections,
                     group='oslo_messaging_zmq')
 
-        kafka_options.register_opts(conf)
+        kafka_options.register_opts(conf, transport_url)
 
         self.config(producer_batch_size=0,
                     group='oslo_messaging_kafka')
@@ -345,7 +347,7 @@ class NotificationFixture(fixtures.Fixture):
         targets.append(oslo_messaging.Target(topic=self.name))
         transport = self.useFixture(TransportFixture(self.conf, self.url))
         self.server = self._get_server(transport, targets)
-        self._ctrl = self.notifier('internal', topic=self.name)
+        self._ctrl = self.notifier('internal', topics=[self.name])
         self._start()
         transport.wait()
 
@@ -369,12 +371,12 @@ class NotificationFixture(fixtures.Fixture):
         if self.thread.isAlive():
             raise Exception("Server did not shutdown properly")
 
-    def notifier(self, publisher, topic=None):
+    def notifier(self, publisher, topics=None):
         transport = self.useFixture(TransportFixture(self.conf, self.url))
         n = notifier.Notifier(transport.transport,
                               publisher,
                               driver='messaging',
-                              topic=topic or self.topics[0])
+                              topics=topics or self.topics)
         transport.wait()
         return n
 
