@@ -153,8 +153,10 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
         super(TestMessagingNotifier, self).setUp()
 
         self.logger = self.useFixture(_ReRaiseLoggedExceptionsFixture()).logger
-        self.stubs.Set(messaging, 'LOG', self.logger)
-        self.stubs.Set(msg_notifier, '_LOG', self.logger)
+        self.useFixture(fixtures.MockPatchObject(
+            messaging, 'LOG', self.logger))
+        self.useFixture(fixtures.MockPatchObject(
+            msg_notifier, '_LOG', self.logger))
 
     @mock.patch('oslo_utils.timeutils.utcnow')
     def test_notifier(self, mock_utcnow):
@@ -212,7 +214,7 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
                 if hasattr(self, 'retry'):
                     send_kwargs['retry'] = self.retry
                 else:
-                    send_kwargs['retry'] = None
+                    send_kwargs['retry'] = -1
                 target = oslo_messaging.Target(topic='%s.%s' % (topic,
                                                                 self.priority))
                 calls.append(mock.call(target,
@@ -226,6 +228,8 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
         uuid.uuid4.assert_called_once_with()
         transport._send_notification.assert_has_calls(calls, any_order=True)
 
+        self.assertTrue(notifier.is_enabled())
+
 TestMessagingNotifier.generate_scenarios()
 
 
@@ -244,7 +248,7 @@ class TestSerializer(test_utils.BaseTestCase):
         notifier = oslo_messaging.Notifier(transport,
                                            'test.localhost',
                                            driver='test',
-                                           topic='test',
+                                           topics=['test'],
                                            serializer=serializer)
 
         message_id = uuid.uuid4()
@@ -269,7 +273,7 @@ class TestSerializer(test_utils.BaseTestCase):
             'timestamp': str(timeutils.utcnow()),
         }
 
-        self.assertEqual([(dict(user='alice'), message, 'INFO', None)],
+        self.assertEqual([(dict(user='alice'), message, 'INFO', -1)],
                          _impl_test.NOTIFICATIONS)
 
         uuid.uuid4.assert_called_once_with()
@@ -295,9 +299,6 @@ class TestNotifierTopics(test_utils.BaseTestCase):
                     group='oslo_messaging_notifications')
         transport = _FakeTransport(self.conf)
 
-        notifier = oslo_messaging.Notifier(transport, 'test.localhost',
-                                           topic='topic1')
-        self.assertEqual(['topic1'], notifier._topics)
         notifier = oslo_messaging.Notifier(transport, 'test.localhost',
                                            topics=['topic1', 'topic2'])
         self.assertEqual(['topic1', 'topic2'], notifier._topics)
@@ -320,6 +321,8 @@ class TestLogNotifier(test_utils.BaseTestCase):
 
         mock_utcnow.return_value = datetime.datetime.utcnow()
 
+        logger = mock.Mock()
+
         message = {
             'message_id': str(message_id),
             'publisher_id': 'test.localhost',
@@ -329,17 +332,19 @@ class TestLogNotifier(test_utils.BaseTestCase):
             'timestamp': str(timeutils.utcnow()),
         }
 
-        logger = mock.Mock()
-        logging.getLogger = mock.Mock()
-        logging.getLogger.return_value = logger
+        with mock.patch.object(logging, 'getLogger') as gl:
+            gl.return_value = logger
 
-        notifier.info({}, 'test.notify', 'bar')
+            notifier.info({}, 'test.notify', 'bar')
+
+            uuid.uuid4.assert_called_once_with()
+            logging.getLogger.assert_called_once_with(
+                'oslo.messaging.notification.test.notify')
 
-        uuid.uuid4.assert_called_once_with()
-        logging.getLogger.assert_called_once_with('oslo.messaging.'
-                                                  'notification.test.notify')
         logger.info.assert_called_once_with(JsonMessageMatcher(message))
 
+        self.assertTrue(notifier.is_enabled())
+
     def test_sample_priority(self):
         # Ensure logger drops sample-level notifications.
         driver = _impl_log.LogDriver(None, None, None)
@@ -348,14 +353,15 @@ class TestLogNotifier(test_utils.BaseTestCase):
                                                   'notification.foo'))
         logger.sample = None
 
-        logging.getLogger = mock.Mock()
-        logging.getLogger.return_value = logger
-
         msg = {'event_type': 'foo'}
-        driver.notify(None, msg, "sample", None)
 
-        logging.getLogger.assert_called_once_with('oslo.messaging.'
-                                                  'notification.foo')
+        with mock.patch.object(logging, 'getLogger') as gl:
+            gl.return_value = logger
+
+            driver.notify(None, msg, "sample", None)
+
+            logging.getLogger.assert_called_once_with('oslo.messaging.'
+                                                      'notification.foo')
 
     def test_mask_passwords(self):
         # Ensure that passwords are masked with notifications
@@ -372,6 +378,31 @@ class TestLogNotifier(test_utils.BaseTestCase):
         logger.info.assert_called_once_with(mask_str)
 
 
+class TestNotificationConfig(test_utils.BaseTestCase):
+
+    def test_retry_config(self):
+        conf = self.messaging_conf.conf
+        self.config(driver=['messaging'],
+                    group='oslo_messaging_notifications')
+
+        conf.set_override('retry', 3, group='oslo_messaging_notifications')
+        transport = _FakeTransport(conf)
+        notifier = oslo_messaging.Notifier(transport)
+
+        self.assertEqual(3, notifier.retry)
+
+    def test_notifier_retry_config(self):
+        conf = self.messaging_conf.conf
+        self.config(driver=['messaging'],
+                    group='oslo_messaging_notifications')
+
+        conf.set_override('retry', 3, group='oslo_messaging_notifications')
+        transport = _FakeTransport(conf)
+        notifier = oslo_messaging.Notifier(transport, retry=5)
+
+        self.assertEqual(5, notifier.retry)
+
+
 class TestRoutingNotifier(test_utils.BaseTestCase):
     def setUp(self):
         super(TestRoutingNotifier, self).setUp()
@@ -382,6 +413,8 @@ class TestRoutingNotifier(test_utils.BaseTestCase):
         self.notifier = oslo_messaging.Notifier(transport)
         self.router = self.notifier._driver_mgr['routing'].obj
 
+        self.assertTrue(self.notifier.is_enabled())
+
     def _fake_extension_manager(self, ext):
         return extension.ExtensionManager.make_test_instance(
             [extension.Extension('test', None, None, ext), ])
@@ -438,10 +471,12 @@ group_2:
             with mock.patch('stevedore.dispatch.DispatchExtensionManager',
                             return_value=self._fake_extension_manager(
                                 mock.MagicMock())):
-                self.router._load_notifiers()
-                groups = list(self.router.routing_groups.keys())
-                groups.sort()
-                self.assertEqual(['group_1', 'group_2'], groups)
+                with mock.patch('oslo_messaging.notify.'
+                                '_impl_routing.LOG'):
+                    self.router._load_notifiers()
+                    groups = list(self.router.routing_groups.keys())
+                    groups.sort()
+                    self.assertEqual(['group_1', 'group_2'], groups)
 
     def test_get_drivers_for_message_accepted_events(self):
         config = r"""
@@ -590,9 +625,25 @@ group_1:
                                config_file):
             with mock.patch('stevedore.dispatch.DispatchExtensionManager',
                             return_value=pm):
-                self.notifier.info({}, 'my_event', {})
-                self.assertFalse(bar_driver.info.called)
-                rpc_driver.notify.assert_called_once_with(
-                    {}, mock.ANY, 'INFO', None)
-                rpc2_driver.notify.assert_called_once_with(
-                    {}, mock.ANY, 'INFO', None)
+                with mock.patch('oslo_messaging.notify.'
+                                '_impl_routing.LOG'):
+
+                    self.notifier.info({}, 'my_event', {})
+                    self.assertFalse(bar_driver.info.called)
+                    rpc_driver.notify.assert_called_once_with(
+                        {}, mock.ANY, 'INFO', -1)
+                    rpc2_driver.notify.assert_called_once_with(
+                        {}, mock.ANY, 'INFO', -1)
+
+
+class TestNoOpNotifier(test_utils.BaseTestCase):
+
+    def test_notifier(self):
+        self.config(driver=['noop'],
+                    group='oslo_messaging_notifications')
+
+        transport = _FakeTransport(self.conf)
+
+        notifier = oslo_messaging.Notifier(transport, 'test.localhost')
+
+        self.assertFalse(notifier.is_enabled())
