@@ -13,9 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-
 from oslo_config import cfg
+from six.moves import mock
 import testscenarios
 
 import oslo_messaging
@@ -340,8 +339,6 @@ class TestSerializer(test_utils.BaseTestCase):
 
         transport._send = mock.Mock()
 
-        msg = dict(method='foo',
-                   args=dict([(k, 's' + v) for k, v in self.args.items()]))
         kwargs = dict(wait_for_reply=True, timeout=None) if self.call else {}
         kwargs['retry'] = None
 
@@ -351,10 +348,13 @@ class TestSerializer(test_utils.BaseTestCase):
         serializer.deserialize_entity = mock.Mock()
         serializer.serialize_context = mock.Mock()
 
-        expected_side_effect = []
-        for arg in self.args:
-            expected_side_effect.append('s' + arg)
-        serializer.serialize_entity.side_effect = expected_side_effect
+        def _stub(ctxt, arg):
+            return 's' + arg
+
+        msg = dict(method='foo', args=dict())
+        for k, v in self.args.items():
+            msg['args'][k] = 's' + v
+        serializer.serialize_entity.side_effect = _stub
 
         if self.call:
             serializer.deserialize_entity.return_value = 'd' + self.retval
