@@ -13,15 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import eventlet
 import threading
+import warnings
 
+import eventlet
+import fixtures
 from oslo_config import cfg
+from six.moves import mock
 import testscenarios
 
-import mock
 import oslo_messaging
 from oslo_messaging import rpc
+from oslo_messaging.rpc import dispatcher
 from oslo_messaging.rpc import server as rpc_server_module
 from oslo_messaging import server as server_module
 from oslo_messaging.tests import utils as test_utils
@@ -109,24 +112,55 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
     def setUp(self):
         super(TestRPCServer, self).setUp(conf=cfg.ConfigOpts())
 
-    def test_constructor(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+    @mock.patch('warnings.warn')
+    def test_constructor(self, warn):
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
+        access_policy = dispatcher.DefaultRPCAccessPolicy
 
-        server = oslo_messaging.get_rpc_server(transport, target, endpoints,
-                                               serializer=serializer)
-
+        warnings.simplefilter("always", FutureWarning)
+        server = oslo_messaging.get_rpc_server(transport,
+                                               target,
+                                               endpoints,
+                                               serializer=serializer,
+                                               access_policy=access_policy)
         self.assertIs(server.conf, self.conf)
         self.assertIs(server.transport, transport)
         self.assertIsInstance(server.dispatcher, oslo_messaging.RPCDispatcher)
         self.assertIs(server.dispatcher.endpoints, endpoints)
         self.assertIs(server.dispatcher.serializer, serializer)
         self.assertEqual('blocking', server.executor_type)
+        self.assertEqual([
+            mock.call("blocking executor is deprecated. Executor default will "
+                      "be removed. Use explicitly threading or eventlet "
+                      "instead in version 'pike' and will be removed in "
+                      "version 'rocky'",
+                      category=FutureWarning, stacklevel=3)
+        ], warn.mock_calls)
+
+    @mock.patch('warnings.warn')
+    def test_constructor_without_explicit_RPCAccessPolicy(self, warn):
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+        target = oslo_messaging.Target(topic='foo', server='bar')
+        endpoints = [object()]
+        serializer = object()
+
+        warnings.simplefilter("always", FutureWarning)
+        oslo_messaging.get_rpc_server(transport, target,
+                                      endpoints, serializer=serializer)
+        self.assertEqual([
+            mock.call(mock.ANY, category=FutureWarning, stacklevel=3),
+            mock.call("blocking executor is deprecated. Executor default will "
+                      "be removed. Use explicitly threading or eventlet "
+                      "instead in version 'pike' and will be removed in "
+                      "version 'rocky'",
+                      category=FutureWarning, stacklevel=3)
+        ], warn.mock_calls)
 
     def test_server_wait_method(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
@@ -158,7 +192,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self.assertEqual(1, listener.cleanup.call_count)
 
     def test_no_target_server(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         server = oslo_messaging.get_rpc_server(
             transport,
@@ -173,7 +207,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def test_no_server_topic(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(server='testserver')
         server = oslo_messaging.get_rpc_server(transport, target, [])
         try:
@@ -185,7 +219,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def _test_no_client_topic(self, call=True):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         client = self._setup_client(transport, topic=None)
 
@@ -206,7 +240,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._test_no_client_topic(call=False)
 
     def test_client_call_timeout(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         finished = False
         wait = threading.Condition()
@@ -234,7 +268,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_unknown_executor(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         try:
             oslo_messaging.get_rpc_server(transport, None, [], executor='foo')
@@ -245,7 +279,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def test_cast(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def __init__(self):
@@ -266,7 +300,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self.assertEqual(['dsfoo', 'dsbar'], endpoint.pings)
 
     def test_call(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
@@ -285,7 +319,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_direct_call(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
@@ -305,7 +339,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_context(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ctxt_check(self, ctxt, key):
@@ -322,7 +356,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_failure(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
@@ -341,8 +375,10 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
                 a = a[0]
             errors.append(str(msg) % a)
 
-        self.stubs.Set(rpc_server_module.LOG, 'debug', stub_debug)
-        self.stubs.Set(rpc_server_module.LOG, 'error', stub_error)
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'debug', stub_debug))
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'error', stub_error))
 
         server_thread = self._setup_server(transport, TestEndpoint())
         client = self._setup_client(transport)
@@ -360,7 +396,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_expected_failure(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         debugs = []
         errors = []
@@ -375,8 +411,10 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
                 a = a[0]
             errors.append(str(msg) % a)
 
-        self.stubs.Set(rpc_server_module.LOG, 'debug', stub_debug)
-        self.stubs.Set(rpc_server_module.LOG, 'error', stub_error)
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'debug', stub_debug))
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'error', stub_error))
 
         class TestEndpoint(object):
             @oslo_messaging.expected_exceptions(ValueError)
@@ -503,9 +541,9 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
         url1 = 'fake:///' + (self.exchange1 or '')
         url2 = 'fake:///' + (self.exchange2 or '')
 
-        transport1 = oslo_messaging.get_transport(self.conf, url=url1)
+        transport1 = oslo_messaging.get_rpc_transport(self.conf, url=url1)
         if url1 != url2:
-            transport2 = oslo_messaging.get_transport(self.conf, url=url1)
+            transport2 = oslo_messaging.get_rpc_transport(self.conf, url=url1)
         else:
             transport2 = transport1
 
