@@ -29,6 +29,7 @@ __all__ = [
 
 import logging
 
+from debtcollector import removals
 from oslo_config import cfg
 import six
 from six.moves.urllib import parse
@@ -76,8 +77,33 @@ class Transport(object):
     This is a mostly opaque handle for an underlying messaging transport
     driver.
 
-    It has a single 'conf' property which is the cfg.ConfigOpts instance used
-    to construct the transport object.
+    RPCs and Notifications may use separate messaging systems that utilize
+    different drivers, access permissions, message delivery, etc. To ensure
+    the correct messaging functionality, the corresponding method should be
+    used to construct a Transport object from transport configuration
+    gleaned from the user's configuration and, optionally, a transport URL.
+
+    The factory method for RPC Transport objects::
+
+        def get_rpc_transport(conf, url=None,
+                              allowed_remote_exmods=None)
+
+    If a transport URL is supplied as a parameter, any transport configuration
+    contained in it takes precedence. If no transport URL is supplied, but
+    there is a transport URL supplied in the user's configuration then that
+    URL will take the place of the URL parameter.
+
+    The factory method for Notification Transport objects::
+
+        def get_notification_transport(conf, url=None,
+                                       allowed_remote_exmods=None)
+
+    If no transport URL is provided, the URL in the notifications section of
+    the config file will be used.  If that URL is also absent, the same
+    transport as specified in the user's default section will be used.
+
+    The Transport has a single 'conf' property which is the cfg.ConfigOpts
+    instance used to construct the transport object.
     """
 
     def __init__(self, driver):
@@ -145,6 +171,33 @@ class DriverLoadFailure(exceptions.MessagingException):
         self.ex = ex
 
 
+def _get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
+    allowed_remote_exmods = allowed_remote_exmods or []
+    conf.register_opts(_transport_opts)
+
+    if not isinstance(url, TransportURL):
+        url = TransportURL.parse(conf, url, aliases)
+
+    kwargs = dict(default_exchange=conf.control_exchange,
+                  allowed_remote_exmods=allowed_remote_exmods)
+
+    try:
+        mgr = driver.DriverManager('oslo.messaging.drivers',
+                                   url.transport.split('+')[0],
+                                   invoke_on_load=True,
+                                   invoke_args=[conf, url],
+                                   invoke_kwds=kwargs)
+    except RuntimeError as ex:
+        raise DriverLoadFailure(url.transport, ex)
+
+    return Transport(mgr.driver)
+
+
+@removals.remove(
+    message='use get_rpc_transport or get_notification_transport'
+)
+@removals.removed_kwarg('aliases',
+                        'Parameter aliases is deprecated for removal.')
 def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
     """A factory method for Transport objects.
 
@@ -166,34 +219,17 @@ def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
 
     :param conf: the user configuration
     :type conf: cfg.ConfigOpts
-    :param url: a transport URL
+    :param url: a transport URL, see :py:class:`transport.TransportURL`
     :type url: str or TransportURL
     :param allowed_remote_exmods: a list of modules which a client using this
                                   transport will deserialize remote exceptions
                                   from
     :type allowed_remote_exmods: list
-    :param aliases: A map of transport alias to transport name
+    :param aliases: DEPRECATED: A map of transport alias to transport name
     :type aliases: dict
     """
-    allowed_remote_exmods = allowed_remote_exmods or []
-    conf.register_opts(_transport_opts)
-
-    if not isinstance(url, TransportURL):
-        url = TransportURL.parse(conf, url, aliases)
-
-    kwargs = dict(default_exchange=conf.control_exchange,
-                  allowed_remote_exmods=allowed_remote_exmods)
-
-    try:
-        mgr = driver.DriverManager('oslo.messaging.drivers',
-                                   url.transport.split('+')[0],
-                                   invoke_on_load=True,
-                                   invoke_args=[conf, url],
-                                   invoke_kwds=kwargs)
-    except RuntimeError as ex:
-        raise DriverLoadFailure(url.transport, ex)
-
-    return Transport(mgr.driver)
+    return _get_transport(conf, url,
+                          allowed_remote_exmods, aliases)
 
 
 class TransportHost(object):
@@ -231,12 +267,40 @@ class TransportURL(object):
 
     Transport URLs take the form::
 
-      transport://user:pass@host:port[,userN:passN@hostN:portN]/virtual_host?query
+      scheme://[user:pass@]host:port[,[userN:passN@]hostN:portN]/virtual_host?query
 
-    i.e. the scheme selects the transport driver, you may include multiple
-    hosts in netloc, the path part is a "virtual host" partition path and
-    the query part contains some driver-specific options which may override
-    corresponding values from a static configuration.
+    where:
+
+    scheme
+      Specifies the transport driver to use. Typically this is `rabbit` for the
+      RabbitMQ broker. See the documentation for other available transport
+      drivers.
+
+    [user:pass@]host:port
+      Specifies the network location of the broker. `user` and `pass` are the
+      optional username and password used for authentication with the broker.
+
+      `user` and `pass` may contain any of the following ASCII characters:
+        * Alphabetic (a-z and A-Z)
+        * Numeric (0-9)
+        * Special characters: & = $ - _ . + ! * ( )
+
+      `user` may include at most one `@` character for compatibility with some
+      implementations of SASL.
+
+      All other characters in `user` and `pass` must be encoded via '%nn'
+
+      You may include multiple different network locations separated by commas.
+      The client will connect to any of the available locations and will
+      automatically fail over to another should the connection fail.
+
+    virtual_host
+      Specifies the "virtual host" within the broker. Support for virtual hosts
+      is specific to the message bus used.
+
+    query
+      Permits passing driver-specific options which override the corresponding
+      values from the configuration file.
 
     :param conf: a ConfigOpts instance
     :type conf: oslo.config.cfg.ConfigOpts
@@ -252,6 +316,8 @@ class TransportURL(object):
     :type query: dict
     """
 
+    @removals.removed_kwarg('aliases',
+                            'Parameter aliases is deprecated for removal.')
     def __init__(self, conf, transport=None, virtual_host=None, hosts=None,
                  aliases=None, query=None):
         self.conf = conf
@@ -282,7 +348,7 @@ class TransportURL(object):
         final_transport = self.aliases.get(transport, transport)
         if not self._deprecation_logged and final_transport != transport:
             # NOTE(sileht): The first step is deprecate this one cycle.
-            # To ensure deployer have updated they configuration during Octavia
+            # To ensure deployer have updated they configuration during Ocata
             # Then in P we will deprecate aliases kwargs of TransportURL() and
             # get_transport() for consuming application
             LOG.warning('legacy "rpc_backend" is deprecated, '
@@ -360,9 +426,12 @@ class TransportURL(object):
 
         return url
 
+    @removals.removed_kwarg('aliases',
+                            'Parameter aliases is deprecated for removal.')
     @classmethod
     def parse(cls, conf, url=None, aliases=None):
-        """Parse an url.
+        """Parse a URL as defined by :py:class:`TransportURL` and return a
+        TransportURL object.
 
         Assuming a URL takes the form of::
 
@@ -399,7 +468,7 @@ class TransportURL(object):
             conf.register_opts(_transport_opts)
         url = url or conf.transport_url
         if not url:
-            return cls(conf, aliases=aliases)
+            return cls(conf) if aliases is None else cls(conf, aliases=aliases)
 
         if not isinstance(url, six.string_types):
             raise InvalidTransportURL(url, 'Wrong URL type')
@@ -432,7 +501,7 @@ class TransportURL(object):
             username = password = port = None
 
             if '@' in host:
-                username, hostname = host.split('@', 1)
+                username, hostname = host.rsplit('@', 1)
                 if ':' in username:
                     username, password = username.split(':', 1)
                     password = parse.unquote(password)
@@ -483,4 +552,7 @@ class TransportURL(object):
                         {'hosts_with_credentials': hosts_with_credentials,
                          'hosts_without_credentials':
                          hosts_without_credentials})
-        return cls(conf, transport, virtual_host, hosts, aliases, query)
+        if aliases is None:
+            return cls(conf, transport, virtual_host, hosts, query=query)
+        else:
+            return cls(conf, transport, virtual_host, hosts, aliases, query)
